#!/usr/bin/env python
# coding: UTF-8

# Copyright (C) 2008 Markus Schmidt
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import sys
import os
import re
import subprocess as sub
import string
import signal
import gobject
import gtk
import datetime, time
import ConfigParser as config
from operator import itemgetter
import languages.lang_en as lang_en
import languages.lang_de as lang_de

import threading
gtk.gdk.threads_init()

try:
 	import pygtk
  	pygtk.require("2.6")
except:
  	pass
try:
	import gtk
  	import gtk.glade
except:
	sys.exit(1)
	
basedir = os.path.dirname(os.path.realpath(__file__))
if not os.path.exists(os.path.join(basedir, "guess.py")):
    if os.path.exists(os.path.join(os.getcwd(), "guess.py")):
        basedir = os.getcwd()
sys.path.insert(0, basedir)

class guess():
	"""This class is a GUI for firing up different applications"""
	
	_TITLE = "Guess"
	_VERSION = open(os.path.join(sys.path[0], 'version')).read()
	_AUTHOR = "Markus Schmidt in 2008\nhttp://mein-neues-blog.de"
	
	_GUESSPATH = os.path.expanduser("~") + "/.guess"
	_CFG_PROGRAMS = "programs.cfg"
	_CFG_SESSIONS = "sessions.cfg"
	_CFG_SETTINGS = "guess.cfg"
	
	_DEFAULT_PROGRAM = { 'name':"", 'start':"", 'stop':"", 'init':"", 'timeout':0, 'priority':50,
						'window':"", 'x':-1, 'y':-1, 'width':-1, 'height':-1, 'desktop':-1, 'timer':0 }
	_DEFAULT_SESSION = { '_name_':"", '_folder_':"" }
	_DEFAULT_SETTINGS = { 'language':"en", 'icon_size':"20", 'close_guess_start':"0", 'close_guess_stop':"0",
					'clear_names':"1", 'default_session_folder':os.path.expanduser("~"), 'timeout':"0.3", "timer_cycle": "2" }
	_LANGUAGE = "en"
	_LANGUAGES = ["en", "de"]
	
	_DEFICON_PROGRAMS = os.path.join(sys.path[0], "pixmaps", "programs.svg")
	_DEFICON_SESSIONS = os.path.join(sys.path[0], "pixmaps", "sessions.svg")
	_DEFICON_FORMATS = ["png", "svg", "xpm"]
	
	_ICON_PATH = [basedir]
	_ICON_PATH.append(os.path.join(basedir, "pixmaps"))
	if os.path.isdir("/usr/share/icons"):
		_ICON_PATH.append("/usr/share/icons")
	if os.path.isdir("/usr/share/pixmaps"):
		_ICON_PATH.append("/usr/share/pixmaps")
	if os.path.isdir(os.path.join(os.path.expanduser("~"), ".pixmaps")):
		_ICON_PATH.append(os.path.join(os.path.expanduser("~"), ".pixmaps"))
	if os.path.isdir(os.path.join(os.path.expanduser("~"), ".icons")):
		_ICON_PATH.append(os.path.join(os.path.expanduser("~"), ".icons"))
	
	_GUI = True
	_SESSION = None
	_RUN = False
	_KILL = False
	_HALT = False
	_COMPIZ = False
	_HELP = False
	
	def __init__(self, argv=sys.argv):
		"""Initiate the program"""
		# Create config file paths
		
		self._CFG_SESSIONS = os.path.join(self._GUESSPATH, self._CFG_SESSIONS)
		self._CFG_PROGRAMS = os.path.join(self._GUESSPATH, self._CFG_PROGRAMS)
		self._CFG_SETTINGS = os.path.join(self._GUESSPATH, self._CFG_SETTINGS)
		# Initiate config objects
		self.config = self.start_config(self._CFG_SETTINGS, self._DEFAULT_SETTINGS, ["Settings"], True)
		self.programs = self.start_config(self._CFG_PROGRAMS, self._DEFAULT_PROGRAM, [], True)
		self.sessions = self.start_config(self._CFG_SESSIONS, self._DEFAULT_SESSION, [], True)

		# Initiate Language
		try:
			ln = gtk.get_default_language()
			i = self._LANGUAGES.index(ln.to_string()[0:2])
			self._LANGUAGE = self._LANGUAGES[i]
		except:
			1	
		if self.config.get("Settings", "language"):
			self._LANGUAGE = self.config.get("Settings", "language")
		self.setLanguage()
	
		# Ctrl+C
		signal.signal(signal.SIGINT, signal.SIG_IGN)
		signal.signal(signal.SIGINT, self.ctrlC)
		
		# Arguments
		c = 1
		lang = None
		for a in sys.argv[1:]:
			# GUI
			if a == "-n" or a == "--no-gui":
				self._GUI = False
			# Sessions
			if a.startswith("--session="):
				self._SESSION = a[10:]
			if a == "-s" and len(sys.argv) > c:
				self._SESSION = sys.argv[c + 1]
			# Start
			if a == "--run" or a == "-r":
				self._RUN = True
			# Stop
			if a == "--kill" or a == "-k":
				self._KILL = True
			# language
			if a.startswith("--language="):
				lang = a[10:]
			if a == "-l" and len(sys.argv) > c:
				lang = sys.argv[c + 1]
			# help
			if a == "-h" or a == "--help":
				self._HELP = True
			c += 1
				
		# language?
		if lang and lang in self._LANGUAGES:
			self._LANGUAGE = lang
			self.setLanguage()
		elif lang:
			self.show_error(self.lang.error_no_valid_language % self._LANGUAGES)
			exit()
		
		# Help?
		if self._HELP:
			print self.lang.shell_help
			exit()
		# invalid session?
		if self._SESSION and not self.sessions.has_section(self._SESSION):
			self.show_error(self.lang.error_no_valid_session % self.sessions.sections())
			exit()
		# run and kill?
		if self._RUN and self._KILL:
			self.show_error(self.lang.error_run_and_kill)
			exit()
		# run or kill but no session?
		if (self._RUN or self._KILL) and not self._SESSION:
			self.show_error(self.lang.error_no_session_action % self.sessions.sections())
			exit()
		# nothing!?
		if not self._RUN and not self._KILL and not self._GUI:
			self.show_error(self.lang.error_nothing_to_do % self.sessions.sections())
			exit()
				
		# start GUI if no argument against was found
		if self._GUI:
			self.start_GUI()
		
		# run session
		if self._RUN and self._SESSION:
			threading.Thread(target=self.run_session, args=(self._SESSION,)).start()
		
		# kill session
		if self._KILL and self._SESSION:
			threading.Thread(target=self.kill_session, args=(self._SESSION,)).start()

################################
# SESSION STARTUP AND SHUTDOWN
################################
	
	def run_session(self, session):
		"""the main thread for starting and setting programs"""
		timeout = float(eval(self.config.get("Settings", "timeout")))
		# first message
		if self._GUI:
			gobject.idle_add(self.update_progress, 0,
						self.lang.progress_title_start % session)
			time.sleep(timeout)
		# compiz found
		if self._GUI and self.get_compiz() == 1:
			gobject.idle_add(self.update_progress, 0,
				self.lang.progress_compiz)
			time.sleep(timeout)
		# get programs
		(progs, jobs, folder) = self.get_programs(session, (0,3))
		if jobs > 0:
			step = float(100/jobs)/100.0
			proc = 0
			# cycle thru sorted progs an exec them if required
			for p in progs:
				if self._HALT is False:
					if p["options"][0] == "1":
						# start, load and sleep
						if self._GUI:
							gobject.idle_add(self.update_progress, proc*step,
								self.lang.progress_starting % p["name"])
						time.sleep(timeout)
						self.start_program(p, folder, proc*step)
						proc += 1
					if p["options"][3] == "1":
						# position, compiz and timer
						if self._GUI:
							gobject.idle_add(self.update_progress, proc*step,
								self.lang.progress_setting % (p["name"] , self.programs.get(p["name"], "window")))
						time.sleep(timeout)
						self.set_program(p)
						proc += 1
						time.sleep(timeout)
				else:
					break
		# cleanup
		if self._GUI:
			gobject.idle_add(self.update_progress, 1,
				self.lang.progress_starting_done % session)
			time.sleep(timeout)
			self.reset_progress()
		self.post_progress(self.config.get("Settings", "close_guess_start"))

	def kill_session(self, session):
		"""the main thread for killing programs"""
		timeout = float(eval(self.config.get("Settings", "timeout")))
		# first message
		if self._GUI:
			gobject.idle_add(self.update_progress, 0,
						self.lang.progress_title_stop % session)
			time.sleep(timeout)
		# get programs
		(progs, jobs, folder) = self.get_programs(session, (1,))
		if jobs > 0:
			step = float(100/jobs)/100.0
			proc = 0			
			# cycle thru sorted progs an exec them if required
			for p in progs:
				if self._HALT is False:
					if p["options"][1] == "1":
						# start, load and sleep
						if self._GUI:
							gobject.idle_add(self.update_progress, proc*step,
								self.lang.progress_stopping % p["name"])
						time.sleep(timeout)
						self.stop_program(p)
						proc += 1
				else:
					break
		# cleanup
		if self._GUI:
			gobject.idle_add(self.update_progress, 1,
				self.lang.progress_stopping_done % session)
			time.sleep(timeout)
			self.reset_progress()
		self.post_progress(self.config.get("Settings", "close_guess_stop"))			
			
	def post_progress(self, close):
		"""post progress: close guess if required"""
		if close == "1":
			if self._GUI:
				gtk.main_quit()
				sys.exit(0)
			else:
				exit()
		
	def start_program(self, p, folder, perc):
		"""build and run a command"""
		cmd = "%s 2>&1 > /dev/null &" % self.create_command(p["name"], folder, p["options"][2])
		os.system(cmd)
		sleep = eval(self.programs.get(p["name"], "timeout"))
		while sleep > 0:
			if self._GUI:
				gobject.idle_add(self.update_progress, perc,
					self.lang.progress_waiting % (sleep, p["name"]))
			time.sleep(1)
			sleep -= 1

	def stop_program(self, p):
		"""build and run a kill-command"""
		cmd = "%s 2>&1 > /dev/null &" % self.programs.get(p["name"], "stop")
		os.system(cmd)

	def set_program(self, p):
		"""build a command for wmctrl and run it"""
		win = self.programs.get(p["name"], "window")
		desk = eval(self.programs.get(p["name"], "desktop"))
		x = eval(self.programs.get(p["name"], "x"))
		y = eval(self.programs.get(p["name"], "y"))
		width = eval(self.programs.get(p["name"], "width"))
		height = eval(self.programs.get(p["name"], "height"))
		sleep = self.config.get("Settings", "timer_cycle")
		if self.get_compiz() == 1 and desk >= 0:
			x = gtk.gdk.screen_width() * desk + x
			desk = -1
		if self.programs.get(p["name"], "timer") != "1":
			cmd = "wmctrl -r \"%s\"" % win
			if desk != -1:
				os.system("%s -t %s 2>&1 > /dev/null &" % cmd, desk)
			os.system("%s -e 0,%s,%s,%s,%s 2>&1 > /dev/null &" % cmd, x, y, width, height)
		else:
			cmd = "%s/timer.py -W \"%s\" -d %s -x %s -y %s -w %s -h %s -s %s" % \
			(sys.path[0], win, desk, x, y, width, height, sleep)
			os.system("%s 2>&1 > /dev/null &" % cmd)
		
	def get_programs(self, session, pos):
		"""return a sorted list of programs and their settings"""
		progs = []
		jobs = 0
		for o in self.sessions.options(session):
			if o == "_name_":
				name = self.sessions.get(session, o)
			elif o == "_folder_":
				folder = self.sessions.get(session, o)
				if folder[-1:] != "/":
					folder = "%s/" % folder
			else:
				opts = self.sessions.get(session, o).split(",")
				j = 0
				for p in pos:
					if opts[p] == "1":
						j += 1
				jobs += j
				if j > 0:
					progs.append({"name":o,
							"priority":eval(self.programs.get(o, "priority")),
							"options":opts})
		return (self.sort_list(progs, "priority"), jobs, folder)

	def get_compiz(self):
		"""search for compiz in active programs"""
		if self._COMPIZ == False:
			ps = os.popen("ps ax -o cmd | grep compiz")
			set_ = 0
			for line in ps.readlines():
				if line.startswith("/usr/bin/compiz.real"):
					self._COMPIZ = set_ = 1
			if set_ == 0:
				self._COMPIZ = 0
		return self._COMPIZ
		
	def sort_list(self, list_, item):
		"""what does it seem to do?"""
		return sorted(list_, key=itemgetter(item))
	
	def create_command(self, prog, folder, load):
		"""build the command with loadfile argument if required"""
		comm = self.programs.get(prog, "start")
		add = self.programs.get(prog, "init")
		if add != "" and load == "1":
			comm = "%s %s" % (comm, add)
		if comm.find("%s") >= 0:
			comm = comm % folder
		return comm		
		 
	def stop_progress(self, widget):
		"""function if a halt is required (not implemented)"""
		self._HALT = True
		return self._HALT
		
	def update_progress(self, proc, label):
		"""set a label and the fraction of status bar"""
		self.window.get_widget("progressStatus").set_fraction(proc)
		self.window.get_widget("labelStatus").set_text(label)			
	
	def reset_progress(self):
		"""reset the status bar"""
		self.window.get_widget("progressStatus").set_fraction(0)
		self.window.get_widget("labelStatus").set_text("")			 
		 
################################
# INIT THE GUI
################################		 
	
	def start_GUI(self):
		"""this class invokes the main jobs to build the window
		like creating missing objects, paths, trees, etc."""
		
		# add standard icon paths to sys


		# create a window and it's corresponding internal objects
		self.create_window()
		self.create_widget_objects()

		#title
		self.window.get_widget("guess").set_title(self._TITLE)
		
		### CREATE MISSING WIDGETS AND GADGETS
		# Programs-Tree
		# generates a treestore for the programs-treviews
		self.programsTree = gtk.TreeStore(
			gtk.gdk.Pixbuf,
			gobject.TYPE_STRING,
			gobject.TYPE_STRING,
			gobject.TYPE_INT,
			gobject.TYPE_BOOLEAN,
			gobject.TYPE_BOOLEAN,
			gobject.TYPE_BOOLEAN,
			gobject.TYPE_BOOLEAN
			)
		# Programs-Programs-Treeview
		self.programsPrograms.set_model(self.programsTree)
		self.programsProgramsSelection = self.programsPrograms.get_selection()
		self.programsProgramsSelection.set_mode(gtk.SELECTION_SINGLE)

		self.add_column_to_tree(self.programsPrograms,
			"", gtk.CellRendererPixbuf(), False, None,
			{"pixbuf":0})
		
		self.add_column_to_tree(self.programsPrograms,
			self.lang.sessions_programs_program, gtk.CellRendererText(), True, 1,
			{"text":1}, "markup=1")
		
		self.add_column_to_tree(self.programsPrograms,
			self.lang.sessions_programs_priority, gtk.CellRendererText(), False, 2,
			{"text":3})
		
		# Sessions-Programs-Treeview
		self.sessionsPrograms.set_model(self.programsTree)
		self.sessionsProgramsSelection = self.sessionsPrograms.get_selection()
		self.sessionsProgramsSelection.set_mode(gtk.SELECTION_NONE)

		self.add_column_to_tree(self.sessionsPrograms,
			"", gtk.CellRendererPixbuf(), True, None,
			{"pixbuf":0})
		
		self.add_column_to_tree(self.sessionsPrograms,
			self.lang.sessions_programs_program, gtk.CellRendererText(), True, 2,
			{"text":1}, "markup=1")
		
		self.add_column_to_tree(self.sessionsPrograms,
			self.lang.sessions_programs_priority, gtk.CellRendererText(), False, 3,
			{"text":3})
		
		self.add_column_to_tree(self.sessionsPrograms,
			self.lang.sessions_programs_start, gtk.CellRendererToggle(), False, None,
			{"active":4}, "", {"activatable":4},
			("toggled", self.toggle_start, self.programsTree))
		
		self.add_column_to_tree(self.sessionsPrograms,
			self.lang.sessions_programs_stop, gtk.CellRendererToggle(), False, None,
			{"active":5}, "", {"activatable":5},
			("toggled", self.toggle_stop, self.programsTree))
		
		self.add_column_to_tree(self.sessionsPrograms,
			self.lang.sessions_programs_init, gtk.CellRendererToggle(), False, None,
			{"active":6}, "", {"activatable":6},
			("toggled", self.toggle_init, self.programsTree))
		
		self.add_column_to_tree(self.sessionsPrograms,
			self.lang.sessions_programs_placement, gtk.CellRendererToggle(), False, None,
			{"active":7}, "", {"activatable":7},
			("toggled", self.toggle_placement, self.programsTree))

		
		self.sessionsPrograms.set_search_column(2)

		# generates a treestore for the sessions-treview
		self.sessionsTree = gtk.TreeStore(
			gtk.gdk.Pixbuf,
			gobject.TYPE_STRING,
			gobject.TYPE_STRING,
			)
		# Sessions-Sessions-Treeview
		self.sessionsSessions.set_model(self.sessionsTree)
		self.sessionsSessionsSelection = self.sessionsSessions.get_selection()
		self.sessionsSessionsSelection.set_mode(gtk.SELECTION_SINGLE)
		
		self.add_column_to_tree(self.sessionsSessions,
			"", gtk.CellRendererPixbuf(), False, None,
			{"pixbuf":0})
		
		self.add_column_to_tree(self.sessionsSessions,
			self.lang.sessions_sessions_session, gtk.CellRendererText(), True, 1,
			{"text":1}, "markup=1")
			
		# Sorting the trees
		self.programsTree.set_sort_column_id(1, gtk.SORT_ASCENDING)
		self.sessionsTree.set_sort_column_id(1, gtk.SORT_ASCENDING)	
		
		# Fill trees
		self.fill_programs_treestore_from_config(self.programsTree, self.programs)
		self.fill_sessions_treestore_from_config(self.sessionsTree, self.sessions)
		
		# Some window related stuff	
		self.fillLabels()
		self.connect_window()
		
		# Language Selector
		for lan in self._LANGUAGES:
			self.settingsLanguage.append_text(lan)
		self.settingsLanguage.set_active(self._LANGUAGES.index(self._LANGUAGE))
		
		# presets
		self.clear_program(None)
		self.clear_session(None)
		self.load_settings()
		
		# preselect session
		if self._SESSION:
			self.select_row_by_name(self.sessionsSessions, self._SESSION)
			self.load_session(None)

		
	def connect_window(self):
		"""set all connections to the signals in the main-window"""
		handlers = {"destroy" : self.destroy,
				"on_notebook_change" : self.change_notebook,
				
				"on_buttonProgramsManual_clicked" : self.show_manual,
				"on_buttonProgramsWizard_clicked" : self.start_wizard,
				"on_buttonProgramsSave_clicked" : self.save_program,
				"on_buttonProgramsRemove_clicked" : self.remove_program,
				"on_buttonProgramsClear_clicked" : self.clear_program,
				
				"on_programsPrograms_row_activated" : self.load_program,
				"on_programsName_changed" : self.program_name_changed,
				
				"on_buttonSessionsSave_clicked" : self.save_session,
				"on_buttonSessionsRemove_clicked" : self.remove_session,
				"on_buttonSessionsStart_clicked" : self.start_session,
				"on_buttonSessionsStop_clicked" : self.stop_session,
				"on_buttonSessionsClear_clicked" : self.clear_session,
				
				"on_sessionsSessions_clicked" : self.load_session,
				"on_sessionsSessions_row_activated" : self.load_session,
				"on_sessionsName_changed" : self.session_name_changed,
				
				"on_buttonSettingsSave_clicked" : self.save_settings,
				"on_settingsClearNames_toggled": self.toggle_clear_names,
				"on_settingsCloseGuessStart_toggled": self.toggle_close_guess_start,
				"on_settingsCloseGuessStop_toggled": self.toggle_close_guess_stop,
				"on_settingsDefaultSessionFolder_changed": self.change_default_session_folder,
				"on_settingsLanguage_changed": self.change_language,
				"on_settingsIconSize_changed": self.change_icon_size,
				"on_settingsTimeout_changed": self.change_timeout,
				"on_settingsTimerCycle_changed": self.change_timer_cycle,
				}
				
		self.window.signal_autoconnect(handlers)
	
	def change_notebook(self, widget):
		"""switch the notebook"""
		1
		
################################
# SETTINGS-OPERATIONS
################################
		
	def toggle_close_guess_start(self, widget):
		self.config.set("Settings", "close_guess_start", "%i" % int(widget.get_active()))
	def toggle_close_guess_stop(self, widget):
		self.config.set("Settings", "close_guess_stop", "%i" % int(widget.get_active()))
	def toggle_clear_names(self, widget):
		self.config.set("Settings", "clear_names", "%i" % int(widget.get_active()))
	def change_default_session_folder(self, widget):
		self.config.set("Settings", "default_session_folder", widget.get_current_folder())
	def change_icon_size(self, widget):
		self.config.set("Settings", "icon_size", "%i" % int(widget.get_value()))
		self.fill_sessions_treestore_from_config(self.sessionsTree, self.sessions)
		self.select_row_by_name(self.sessionsSessions, self.sessionsName)
		self.fill_programs_treestore_from_config(self.programsTree, self.programs)
		self.select_row_by_name(self.programsPrograms, self.programsName)
	def change_timeout(self, widget):
		self.config.set("Settings", "timeout", "%f" % float(widget.get_value()))
	def change_timer_cycle(self, widget):
		self.config.set("Settings", "timer_cycle", "%i" % int(widget.get_value()))
	def save_settings(self, widget):
		self.config.write(open(self._CFG_SETTINGS, "w"))
	
	def load_settings(self):
		self.settingsCloseGuessStart.set_active(eval(self.config.get("Settings", "close_guess_start")))
		self.settingsCloseGuessStop.set_active(eval(self.config.get("Settings", "close_guess_stop")))
		self.settingsClearNames.set_active(eval(self.config.get("Settings", "clear_names")))
		self.settingsDefaultSessionFolder.set_current_folder(self.config.get("Settings", "default_session_folder"))
		self.settingsIconSize.set_value(eval(self.config.get("Settings", "icon_size")))
		self.settingsTimeout.set_value(eval(self.config.get("Settings", "timeout")))
		self.settingsTimerCycle.set_value(eval(self.config.get("Settings", "timer_cycle")))
						
	def change_language(self, widget):
		"""handler for settingsLanguage"""
		model = self.settingsLanguage.get_model()
		self._LANGUAGE = model.get_value(self.settingsLanguage.get_active_iter(), 0)
		self.config.set("Settings", "language", self._LANGUAGE)
		self.setLanguage() 
		self.fillLabels()
		
################################
# SESSION-OPERATIONS
################################

	def start_session(self, widget):
		self._SESSION = self.sessionsName.get_text()
		if self._SESSION in self.sessions.sections():
			threading.Thread(target=self.run_session, args=(self._SESSION,)).start()
		else:
			self.show_error(self.lang.error_no_valid_session % self.sessions.sections())
	def stop_session(self, widget):
		self._SESSION = self.sessionsName.get_text()
		if self._SESSION in self.sessions.sections():
			threading.Thread(target=self.kill_session, args=(self._SESSION,)).start()
		else:
			self.show_error(self.lang.error_no_valid_session % self.sessions.sections())
	def toggle_start(self, cell, path, model):
		model[path][4] = not model[path][4]
	def toggle_stop(self, cell, path, model):
		model[path][5] = not model[path][5]
	def toggle_init(self, cell, path, model):
		model[path][6] = not model[path][6]
	def toggle_placement(self, cell, path, model):
		model[path][7] = not model[path][7]
		
	def load_session(self, widget):
		"""if the user selects a session by clicking in treeview"""
		(model, pos) = self.sessionsSessionsSelection.get_selected()
		if pos:
			name = self.sessionsTree.get_value(pos, 2)
			self.fill_session(name)
			
	def save_session(self, widget):
		name = self.sessionsName.get_text()
		if name == "":
			info = self.message_dialog(self.lang.warning_no_session_name,
				self.lang.warning_message_title, gtk.MESSAGE_INFO, gtk.BUTTONS_OK)
		elif name == "_name_" or name == "_folder_":
			info = self.message_dialog(self.lang.warning_restricted_session_name,
				self.lang.warning_message_title, gtk.MESSAGE_INFO, gtk.BUTTONS_OK)
		elif self.sessions.has_section(name):
			self.save_session_options_to_sessions(name)
		else:
			self.sessions.add_section(name)
			self.save_session_options_to_sessions(name)			
		self.sessions.write(open(self._CFG_SESSIONS, "w"))
		self.fill_sessions_treestore_from_config(self.sessionsTree, self.sessions)
		self.select_row_by_name(self.sessionsSessions, name)
	
	def remove_session(self, widget):
		(model, pos) = self.sessionsSessionsSelection.get_selected()
		if pos:
			name = self.sessionsTree.get_value(pos, 2)
			self.sessions.remove_section(name)
			self.sessions.write(open(self._CFG_SESSIONS, "w"))
			self.fill_sessions_treestore_from_config(self.sessionsTree, self.sessions)
			self.clear_session(None)
		else:
			info = self.message_dialog(self.lang.warning_no_session_selection,
				self.lang.warning_message_title, gtk.MESSAGE_INFO, gtk.BUTTONS_OK)
	
	def clear_session(self, widget):
		if self.config.get("Settings", "clear_names"):
			self.sessionsName.set_text(self._DEFAULT_SESSION["_name_"])
		self.sessionsFolder.set_current_folder(self.config.get("Settings", "default_session_folder"))
		i = self.programsTree.get_iter_first()
		while i:
			self.programsTree.set_value(i, 4, False)
			self.programsTree.set_value(i, 5, False)
			self.programsTree.set_value(i, 6, False)
			self.programsTree.set_value(i, 7, False)
			i = self.programsTree.iter_next(i)
					
	def save_session_options_to_sessions(self, name):
		self.sessions.set(name, "_name_", name)
		self.sessions.set(name, "_folder_", self.sessionsFolder.get_current_folder())
		i = self.programsTree.get_iter_first()
		while i:
			prog = self.programsTree.get_value(i, 2)
			self.sessions.set(name, prog, "%i,%i,%i,%i" %
			(int(self.programsTree.get_value(i, 4)),
			int(self.programsTree.get_value(i, 5)),
			int(self.programsTree.get_value(i, 6)),
			int(self.programsTree.get_value(i, 7)))
			)
			i = self.programsTree.iter_next(i)
					
	def session_name_changed(self, widget):
		"""check if we have to select an entry in tree"""
		name = self.sessionsName.get_text()
		self.select_row_by_name(self.sessionsSessions, name, True)

	def fill_session(self, name):
		"""fill the entries with values from programs config"""
		self.sessionsName.set_text(self.sessions.get(name, "_name_"))
		self.sessionsFolder.set_current_folder(self.sessions.get(name, "_folder_"))
		i = self.programsTree.get_iter_first()
		while i:
			prog = self.programsTree.get_value(i, 2)
			if self.sessions.has_option(name, prog):
				settings = self.sessions.get(name, prog)
				settings = settings.split(",")
				self.programsTree.set_value(i, 4, bool(int(settings[0])))
				self.programsTree.set_value(i, 5, bool(int(settings[1])))
				self.programsTree.set_value(i, 6, bool(int(settings[2])))
				self.programsTree.set_value(i, 7, bool(int(settings[3])))
			else:
				self.programsTree.set_value(i, 4, False)
				self.programsTree.set_value(i, 5, False)
				self.programsTree.set_value(i, 6, False)
				self.programsTree.set_value(i, 7, False)
			i = self.programsTree.iter_next(i)
			

################################
# PROGRAM-OPERATIONS
################################
		
	def load_program(self, widget):
		"""if the user selects an app by clicking in treeview"""
		(model, pos) = self.programsProgramsSelection.get_selected()
		if pos:
			name = self.programsTree.get_value(pos, 2)
			self.fill_program(name)

	def save_program(self, widget):
		name = self.programsName.get_text()
		if name == "":
			info = self.message_dialog(self.lang.warning_no_program_name,
				self.lang.warning_message_title, gtk.MESSAGE_INFO, gtk.BUTTONS_OK)
		else:
			if self.programs.has_section(name):
				self.save_program_options_to_programs(name)
			else:
				self.programs.add_section(name)
				self.save_program_options_to_programs(name)			
			self.programs.write(open(self._CFG_PROGRAMS, "w"))
			self.fill_programs_treestore_from_config(self.programsTree, self.programs)
			self.select_row_by_name(self.programsPrograms, name)
			if self.sessionsName.get_text() != "":
				self.fill_session(self.sessionsName.get_text())
		
	def remove_program(self, widget):
		(model, pos) = self.programsProgramsSelection.get_selected()
		(model1, pos1) = self.sessionsSessionsSelection.get_selected()
		if pos1:
			sess = self.sessionsTree.get_value(pos1, 2)
		#	print sess
		if pos:
			name = self.programsTree.get_value(pos, 2)
			# remove from programs_cfg
			self.programs.remove_section(name)
			self.programs.write(open(self._CFG_PROGRAMS, "w"))
			self.fill_programs_treestore_from_config(self.programsTree, self.programs)
			# remove from sessions cfg
			for s in self.sessions.sections():
				if self.sessions.has_option(s, name):
					self.sessions.remove_option(s, name)
			self.sessions.write(open(self._CFG_SESSIONS, "w"))
			self.clear_program(None)
			if pos1:
				self.select_row_by_name(self.sessionsSessions, sess)
				self.fill_session(sess)
		else:
			info = self.message_dialog(self.lang.warning_no_program_selection,
				self.lang.warning_message_title, gtk.MESSAGE_INFO, gtk.BUTTONS_OK)
	
	def clear_program(self, widget):
		for i in self._DEFAULT_PROGRAM:
			if i != "name" or self.config.get("Settings", "clear_names"):
				obj = eval("self.programs" + i.title())
				try:
					obj.set_value(self._DEFAULT_PROGRAM[i])
				except:
					try:
						obj.set_text(self._DEFAULT_PROGRAM[i])
					except:
						obj.set_active(self._DEFAULT_PROGRAM[i])
					
	def save_program_options_to_programs(self, name):
		for i in self._DEFAULT_PROGRAM:
			obj = eval("self.programs" + i.title())
			try:
				value = "%i" % obj.get_value()
			except:
				try:
					value = obj.get_text()
				except:
					value = "%i" % int(obj.get_active())
			self.programs.set(name, i, value)
		
	def program_name_changed(self, widget):
		"""check if we have to select an entry in tree"""
		name = self.programsName.get_text()
		self.select_row_by_name(self.programsPrograms, name, True)

	def fill_program(self, name):
		"""fill the entries with values from programs config"""
		self.programsName.set_text(self.programs.get(name, "name"))
		self.programsStart.set_text(self.programs.get(name, "start"))
		self.programsStop.set_text(self.programs.get(name, "stop"))
		self.programsInit.set_text(self.programs.get(name, "init"))
		self.programsTimeout.set_value(eval(self.programs.get(name, "timeout")))
		self.programsPriority.set_value(eval(self.programs.get(name, "priority")))
		self.programsWindow.set_text(self.programs.get(name, "window"))
		self.programsDesktop.set_value(eval(self.programs.get(name, "desktop")))
		self.programsWidth.set_value(eval(self.programs.get(name, "width")))
		self.programsHeight.set_value(eval(self.programs.get(name, "height")))
		self.programsX.set_value(eval(self.programs.get(name, "x")))
		self.programsY.set_value(eval(self.programs.get(name, "y")))
		self.programsTimer.set_active(eval(self.programs.get(name, "timer")))
		
	def format_program_command(self, command):
		"""extract a program name from a command line"""
		command = command.strip()
		if command.find(" ") > -1:
			command = command[0:command.find(" ")]
		return command

#######################################
# TREEVIEW-OPERATIONS
#######################################		

	def fill_sessions_treestore_from_config(self, treestore, config):
		"""clear and refill the textview"""
		treestore.clear()
		for s in config.sections():
			self.append_sessions_treestore_row(
				treestore,
				None,
				self.get_icon([self.sessions.get(s, "_folder_")] + self._ICON_PATH,
							[config.get(s, "_name_")[config.get(s, "_name_").rfind(".")+1:], "guess"],
							self._DEFICON_FORMATS,
							self._DEFICON_SESSIONS),
				config.get(s, "_name_"))

	def append_sessions_treestore_row(self, treestore, position, imagefile, name):
		"""inserts a new column in the treestore"""
		s = eval(self.config.get("Settings", "icon_size"))
		image = gtk.gdk.pixbuf_new_from_file_at_size(imagefile, s, s)
		#higher quality:
		#im = gtk.gdk.pixbuf_new_from_file(imagefile)
		#image = image.scale_simple(s, s, gtk.gdk.INTERP_BILINEAR)
		treestore.append(position, [image, "<b>%s</b>" % name, name]) 
					
	def fill_programs_treestore_from_config(self, treestore, config):
		"""clear and refill the textview"""
		treestore.clear()
		for s in config.sections():
			self.append_programs_treestore_row(
				treestore,
				None,
				self.get_icon(self._ICON_PATH,
							[config.get(s, "name")[config.get(s, "name").rfind(".")+1:]],
							self._DEFICON_FORMATS,
							self._DEFICON_PROGRAMS),
				config.get(s, "name"),
				config.get(s, "priority"))
				
	def append_programs_treestore_row(self, treestore, position, imagefile, name, priority):
		"""inserts a new column in the treestore"""
		s = eval(self.config.get("Settings", "icon_size"))
		image = gtk.gdk.pixbuf_new_from_file_at_size(imagefile, s, s)
		#higher quality:
		#im = gtk.gdk.pixbuf_new_from_file(imagefile)
		#image = image.scale_simple(s, s, gtk.gdk.INTERP_BILINEAR)
		treestore.append(position, [image, "<b>%s</b>" % name, name, eval(priority),
						False, False, False, False]) 
		
	def select_row_by_name(self, treeview, name, unselect=False):
		"""if name matches one of the entires in treeview,
		select it. otherwise unselect all"""
		selection = treeview.get_selection()
		model = treeview.get_model()
		sel = False
		i = model.get_iter_first()
		while i:
			if model.get_value(i, 2) == name:
				selection.select_iter(i)
				sel = i
				break
			i = model.iter_next(i)
		if not sel:
			if unselect:
				selection.unselect_all()
			return None
		else:
			return sel

	def add_column_to_tree(self, tree, title, item, resize=False,
				sort=None, addattr={}, setattr_=None, setprop={}, connect=None, resizable=True):
		column = gtk.TreeViewColumn(title)
		cell = item
		column.pack_start(cell, resize)
		for i in addattr:
			column.add_attribute(cell, i, addattr[i])
		if setattr_:
			eval("column.set_attributes(cell, %s)" % setattr_)
		if sort is not None:
			column.set_sort_column_id(sort)
		if connect is not None:
			cell.connect(connect[0], connect[1], connect[2])
		for i in setprop:
			cell.set_property(i, setprop[i])
		tree.append_column(column)
		if resizable:
			column.set_resizable(True)
		return column

#######################################
# ICON-OPERATIONS
#######################################
	
	def get_icon(self, path, name, suffix, default):
		for p in path:
			for n in name:
				for s in suffix:
					if os.path.exists(os.path.join(p, "%s.%s" % (n, s))):
						return (os.path.join(p, "%s.%s" % (n, s)))
					elif os.path.exists(os.path.join(p, "%s.%s" % (n.lower(), s))):
						return (os.path.join(p, "%s.%s" % (n.lower(), s)))
		return default

#######################################
# CONFIG-OPERATIONS
#######################################

	def start_config(self, cfgfile, defaults={}, sections=[], optionxform_off = False):
		if not os.path.exists(os.path.dirname(cfgfile)):
			os.mkdir(os.path.dirname(cfgfile))
		if not os.path.exists(cfgfile):
			c = open(cfgfile, "w")
			c.close()
		cfg = config.RawConfigParser(defaults)
		if optionxform_off:
			cfg.optionxform = str
		cfg.readfp(open(cfgfile))
		for s in sections:
			if not cfg.has_section(s):
				cfg.add_section(s)
				for i in defaults:
					cfg.set(s, i, defaults[i])
		return cfg
		
#######################################
# RTFM
#######################################
			
	def show_manual(self, widget):
		"""grab and display a manual page"""
		if self.programsStart.get_text() != "":
			prog = self.format_program_command(self.programsStart.get_text())
			man = os.popen("man " + prog + " | col -bf")
			output = ""
			while 1:
				line = man.readline()
				if not line: break
				output += line
			if output == "":
				self.message_dialog(self.lang.error_no_manual % prog,
					self.lang.error_message_title, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK)
			else:
				win = self.get_manual_window()	
				win[0].get_widget("manual").set_title("Manual " + prog)
				win[1].set_text(output)
		else:
			self.message_dialog(self.lang.error_no_program_name,
					self.lang.error_message_title, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK)
		
	def get_manual_window(self):
		"""create a manual window and a text-object"""
		manual = gtk.glade.XML(self.gladefile, "manual",)
		mantext = gtk.TextBuffer()
		manual.get_widget("textviewManual").set_buffer(mantext)
		return manual, mantext	
	
			
#######################################
# WIZARD
#######################################

	def start_wizard(self, widget):
		"muahahahaaa the goblin"""
		self.wizard()

	def wizard(self):
		"""try to get the values from xprop, xwininfo and ps"""
		x = None
		y = None
		width = None
		height = None
		id_ = None
		pid = None
		name = None
		window = None
		desktop = None
		cmd = None
		
		xwininfo = os.popen("xwininfo")
		while 1:
			line = xwininfo.readline()
			if not line: break
			if line.startswith("xwininfo: Window id: ") != False:
				id_ = re.search(r"0x[0-9a-fA-F]+", line).group()
				window = re.search(r"\".*?\"", line).group()[1:-1]
			if line.startswith("  Absolute upper-left X:") != False:
				x = re.search(r"[0-9]+", line).group()
			if line.startswith("  Absolute upper-left Y:") != False:
				y = re.search(r"[0-9]+", line).group()	
			if line.startswith("  Width:") != False:
				width = re.search(r"[0-9]+", line).group()
			if line.startswith("  Height:") != False:
				height = re.search(r"[0-9]+", line).group()
		
		if id_ is not None:
			xprop = os.popen("xprop -id %s" % id_)
			while 1:
				line = xprop.readline()
				if not line: break	
				if line.startswith("_NET_WM_PID") != False:
					pid = re.search(r"[0-9]+", line).group()
				if line.startswith("_NET_WM_DESKTOP") != False:
					desktop = re.search(r"[0-9]+", line).group()
				if line.startswith("WM_CLASS") != False:
					name = re.search(r".*\"(.*?)\"", line).group(1)
					
		if pid is not None:	
			ps = os.popen("ps -o cmd -p %s" % pid)
			while 1:
				line = ps.readline()
				if not line: break	
				cmd = line.strip()
		
		if cmd is not None:
			self.programsStart.set_text(cmd)
			self.programsStop.set_text("pkill -f \"%s\"" % cmd)
		if window is not None:
			self.programsWindow.set_text(window)
		if x is not None:
			self.programsX.set_value(eval(x))
			self.programsX.update()
		if y is not None:
			self.programsY.set_value(eval(y))
			self.programsY.update()
		if width is not None:
			self.programsWidth.set_value(eval(width))
			self.programsWidth.update()
		if height is not None:
			self.programsHeight.set_value(eval(height))
			self.programsHeight.update()
		if desktop is not None:
			self.programsDesktop.set_value(eval(desktop))
			self.programsDesktop.update()
		if name is not None:
			self.programsName.set_text(name)
					
				
#######################################
# WINDOW-FUNCTIONS
#######################################
	
	def create_window(self):
		"""create the main window"""
		self.gladefile = sys.path[0] + "/guess.glade"  
		self.window = gtk.glade.XML(self.gladefile, "guess",)
		
	def create_widget_objects(self):
		"""here we wrap all widgets to local objects for easier calls"""
		self.sessionsSessions = self.window.get_widget("sessionsSessions")
		self.sessionsName = self.window.get_widget("sessionsName")
		self.sessionsFolder = self.window.get_widget("sessionsFolder")	
		self.sessionsPrograms = self.window.get_widget("sessionsPrograms")
		
		self.buttonSessionsSave = self.window.get_widget("buttonSessionsSave")
		self.buttonSessionsRemove = self.window.get_widget("buttonSessionsRemove")
		self.buttonSessionsStart = self.window.get_widget("buttonSessionsStart")
		self.buttonSessionsStop = self.window.get_widget("buttonSessionsStop")
		
		self.programsPrograms = self.window.get_widget("programsPrograms")
		self.programsName = self.window.get_widget("programsName")
		self.programsStart = self.window.get_widget("programsStart")
		self.programsStop = self.window.get_widget("programsStop")
		self.programsInit = self.window.get_widget("programsInit")
		self.programsTimeout = self.window.get_widget("programsTimeout")
		self.programsWindow = self.window.get_widget("programsWindow")
		self.programsDesktop = self.window.get_widget("programsDesktop")
		self.programsX = self.window.get_widget("programsX")
		self.programsY = self.window.get_widget("programsY")
		self.programsWidth = self.window.get_widget("programsWidth")
		self.programsHeight = self.window.get_widget("programsHeight")
		self.programsTimer = self.window.get_widget("programsTimer")
		self.programsPriority = self.window.get_widget("programsPriority")
		
		self.buttonProgramsSave = self.window.get_widget("buttonProgramsSave")
		self.buttonProgramsRemove = self.window.get_widget("buttonProgramsRemove")
		
		self.settingsLanguage = self.window.get_widget("settingsLanguage")
		self.settingsCloseGuessStart = self.window.get_widget("settingsCloseGuessStart")
		self.settingsCloseGuessStop = self.window.get_widget("settingsCloseGuessStop")
		self.settingsClearNames = self.window.get_widget("settingsClearNames")
		self.settingsDefaultSessionFolder = self.window.get_widget("settingsDefaultSessionFolder")
		self.settingsIconSize = self.window.get_widget("settingsIconSize")
		self.settingsTimeout = self.window.get_widget("settingsTimeout")
		self.settingsTimerCycle = self.window.get_widget("settingsTimerCycle")
				
	def show_error(self, error):
		if self._GUI:
			errorwin = self.message_dialog(error,
				self.lang.error_message_title, gtk.MESSAGE_INFO, gtk.BUTTONS_OK)
		else:
			print self.lang.error_message_shell % error

	def show_warning(self, warning):
		if self._GUI:
			warningwin = self.message_dialog(warning,
				self.lang.warning_message_title, gtk.MESSAGE_INFO, gtk.BUTTONS_OK)
		else:
			print self.lang.warning_message_shell % error
		
	def message_dialog(self, message, title = "guess-Message",
					behavior = gtk.MESSAGE_INFO, buttons = gtk.BUTTONS_OK):
		"""creates a message dialog with variable tile, message, behavior and buttons"""
		dialog = gtk.MessageDialog(
			parent=None,
			flags=gtk.DIALOG_DESTROY_WITH_PARENT,
			type=behavior,
			buttons=buttons,
			message_format=message)
		dialog.set_title(title)
		dialogreturn = dialog.run()
		dialog.destroy()
		return dialogreturn		

	def destroy(self, widget):
		"""wtf?!1"""
		gtk.main_quit()

	def ctrlC(self, signum, frame):
		self.destroy(None)

	def setLanguage(self):
		"""changes the language object to internal language"""
		self.lang = eval("lang_" + self._LANGUAGE + ".guessLang")()

	def fillLabels(self):
		"""Fill al the labels with language depending text"""
		self.window.get_widget("labelSessionsName").set_text(self.lang.session_name_label)
		self.window.get_widget("labelSessionsFolder").set_text(self.lang.session_folder_label)
		self.window.get_widget("labelSessionsPrograms").set_text(self.lang.session_programs_label)
		
		self.window.get_widget("labelProgramsName").set_text(self.lang.program_name_label)
		self.window.get_widget("labelProgramsStart").set_text(self.lang.program_start_label)
		self.window.get_widget("labelProgramsStop").set_text(self.lang.program_stop_label)
		self.window.get_widget("labelProgramsInit").set_text(self.lang.program_init_label)
		self.window.get_widget("labelProgramsTimeout").set_text(self.lang.program_timeout_label)
		self.window.get_widget("labelProgramsWindow").set_text(self.lang.program_window_label)
		self.window.get_widget("labelProgramsDesktop").set_text(self.lang.program_desktop_label)
		self.window.get_widget("labelProgramsX").set_text(self.lang.program_x_label)
		self.window.get_widget("labelProgramsY").set_text(self.lang.program_y_label)
		self.window.get_widget("labelProgramsWidth").set_text(self.lang.program_width_label)
		self.window.get_widget("labelProgramsHeight").set_text(self.lang.program_height_label)
		self.window.get_widget("labelProgramsTimer").set_text(self.lang.program_timer_label)
		self.window.get_widget("labelProgramsPriority").set_text(self.lang.program_priority_label)
		
		self.window.get_widget("labelProgramsProgram").set_markup(self.lang.programs_program)		
		self.window.get_widget("labelProgramsPlacement").set_markup(self.lang.programs_placement)
		
		self.window.get_widget("labelSettingsCloseGuessStart").set_text(self.lang.settings_close_guess_start)
		self.window.get_widget("labelSettingsCloseGuessStop").set_text(self.lang.settings_close_guess_stop)
		self.window.get_widget("labelSettingsClearNames").set_text(self.lang.settings_clear_names)
		self.window.get_widget("labelSettingsDefaultSessionFolder").set_text(self.lang.settings_default_session_folder)
		self.window.get_widget("labelSettingsLanguage").set_text(self.lang.settings_language)
		self.window.get_widget("labelSettingsIconSize").set_text(self.lang.settings_icon_size)
		self.window.get_widget("labelSettingsTimeout").set_text(self.lang.settings_timeout)
		self.window.get_widget("labelSettingsTimerCycle").set_text(self.lang.settings_timer_cycle)

		self.window.get_widget("labelSettingsCredits").set_markup(self.lang.credits % (self._TITLE, self._VERSION, self._AUTHOR))
				
		self.window.get_widget("labelNotebookSessions").set_text(self.lang.sessions_notebook)
		self.window.get_widget("labelNotebookPrograms").set_text(self.lang.programs_notebook)
		self.window.get_widget("labelNotebookSettings").set_text(self.lang.settings_notebook)	
		
		# TOOLTIPS
		self.window.get_widget("labelSessionsName").set_tooltip_text(self.lang.session_name_tt)
		self.window.get_widget("labelSessionsFolder").set_tooltip_text(self.lang.session_folder_tt)
		self.window.get_widget("labelSessionsPrograms").set_tooltip_text(self.lang.session_programs_tt)
		self.window.get_widget("buttonSessionsSave").set_tooltip_text(self.lang.session_save_tt)
		self.window.get_widget("buttonSessionsClear").set_tooltip_text(self.lang.session_clear_tt)
		self.window.get_widget("buttonSessionsRemove").set_tooltip_text(self.lang.session_remove_tt)
		self.window.get_widget("buttonSessionsStart").set_tooltip_text(self.lang.session_start_tt)
		self.window.get_widget("buttonSessionsStop").set_tooltip_text(self.lang.session_stop_tt)
				
		self.window.get_widget("labelProgramsName").set_tooltip_text(self.lang.program_name_tt)
		self.window.get_widget("labelProgramsStart").set_tooltip_text(self.lang.program_start_tt)
		self.window.get_widget("labelProgramsStop").set_tooltip_text(self.lang.program_stop_tt)
		self.window.get_widget("labelProgramsInit").set_tooltip_text(self.lang.program_init_tt)
		self.window.get_widget("labelProgramsTimeout").set_tooltip_text(self.lang.program_timeout_tt)
		self.window.get_widget("labelProgramsWindow").set_tooltip_text(self.lang.program_window_tt)
		self.window.get_widget("labelProgramsDesktop").set_tooltip_text(self.lang.program_desktop_tt)
		self.window.get_widget("labelProgramsX").set_tooltip_text(self.lang.program_x_tt)
		self.window.get_widget("labelProgramsY").set_tooltip_text(self.lang.program_y_tt)
		self.window.get_widget("labelProgramsWidth").set_tooltip_text(self.lang.program_width_tt)
		self.window.get_widget("labelProgramsHeight").set_tooltip_text(self.lang.program_height_tt)
		self.window.get_widget("labelProgramsTimer").set_tooltip_text(self.lang.program_timer_tt)
		self.window.get_widget("labelProgramsPriority").set_tooltip_text(self.lang.program_priority_tt)
		self.window.get_widget("buttonProgramsWizard").set_tooltip_text(self.lang.program_wizard_tt)
		self.window.get_widget("buttonProgramsManual").set_tooltip_text(self.lang.program_manual_tt)
		self.window.get_widget("buttonProgramsSave").set_tooltip_text(self.lang.program_save_tt)
		self.window.get_widget("buttonProgramsClear").set_tooltip_text(self.lang.program_clear_tt)
		self.window.get_widget("buttonProgramsRemove").set_tooltip_text(self.lang.program_remove_tt)
			
		self.window.get_widget("labelSettingsCloseGuessStart").set_tooltip_text(self.lang.settings_close_guess_start_tt)
		self.window.get_widget("labelSettingsCloseGuessStop").set_tooltip_text(self.lang.settings_close_guess_stop_tt)
		self.window.get_widget("labelSettingsClearNames").set_tooltip_text(self.lang.settings_clear_names_tt)
		self.window.get_widget("labelSettingsLanguage").set_tooltip_text(self.lang.settings_language_tt)
		self.window.get_widget("labelSettingsDefaultSessionFolder").set_tooltip_text(self.lang.settings_default_session_folder_tt)
		self.window.get_widget("labelSettingsIconSize").set_tooltip_text(self.lang.settings_icon_size_tt)
		self.window.get_widget("labelSettingsTimeout").set_tooltip_text(self.lang.settings_timeout_tt)
		self.window.get_widget("labelSettingsTimerCycle").set_tooltip_text(self.lang.settings_timer_cycle_tt)
		self.window.get_widget("labelSettingsCredits").set_tooltip_text(self.lang.settings_credits_tt)
		self.window.get_widget("buttonSettingsSave").set_tooltip_text(self.lang.settings_button_save_tt)
						
		self.window.get_widget("labelNotebookSessions").set_tooltip_text(self.lang.sessions_tt)
		self.window.get_widget("labelNotebookPrograms").set_tooltip_text(self.lang.programs_tt)	
		self.window.get_widget("labelNotebookSettings").set_tooltip_text(self.lang.settings_tt)	

	
def main_quit(obj):
	"""main_quit function, it stops the thread and the gtk's main loop"""
	gtk.main_quit()
			
#######################################
# EVERYTHING HAS TO COME TO AN END
#######################################			
			
if __name__ =="__main__":
	guessWindow = guess()
	gtk.main()

